class credentials:
    EMAIL_ID = ''
    EMAIL_PASS = ''
    EMAIL_USER = ''
    SLACK_TOKEN = ''

    def __init__(self):
        pass


def _set_email_credentials(cred_obj, email, password, name):
    cred_obj.EMAIL_ID = email
    cred_obj.EMAIL_PASS = password
    cred_obj.EMAIL_USER = name


def _set_slack_token_credentials(cred_obj, token):
    cred_obj.SLACK_TOKEN = token



from email.message import Message
from .credential_handler import _set_email_credentials
from .errors import GmailLoginException, RecipientNotPresentException
from .async_decorator import Async
from notipyer import _get_creds
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.base import MIMEBase
from email import encoders
import smtplib
import os

SMTP_GMAIL_URL = 'smtp.gmail.com'

mail_cred = _get_creds()


def _check_valid_string(string):
    if len(string) > 0:
        return True
    return False


def set_email_config(email, password, sender_name=''):
    global mail_cred
    if _check_valid_string(email) and _check_valid_string(password):
        _set_email_credentials(mail_cred, email, password, sender_name)


def _send_email_helper(subject, message, to_addr=None, cc_addr=None, bcc_addr=None, attachment_path=None, html_text=None):
    global mail_cred
    global SMTP_GMAIL_URL

    to_addr, cc_addr, bcc_addr = _check_recipients(to_addr, cc_addr, bcc_addr)

    client = smtplib.SMTP_SSL(SMTP_GMAIL_URL)
    client = _login_client(client)
    recipients, email_body = _build_email(subject, message, to_addr, cc_addr, bcc_addr, attachment_path, html_text=html_text)
    client.sendmail(mail_cred.EMAIL_ID, recipients, email_body)
    client.quit()
    return True


def send_email(subject, message, to_addr=None, cc_addr=None, bcc_addr=None, attachment_path=None, html_text=None, is_async=True):
    global _send_email_helper
    if is_async:
        _send_email_helper = Async(_send_email_helper)

    return _send_email_helper(subject, message, to_addr, cc_addr, bcc_addr, attachment_path, html_text)


def _login_client(client):
    try:
        client.login(mail_cred.EMAIL_ID, mail_cred.EMAIL_PASS)
        return client
    except smtplib.SMTPAuthenticationError:
        raise GmailLoginException()


def _check_recipients(to_addr, cc_addr, bcc_addr):
    if to_addr is None:
        to_addr = []
    if cc_addr is None:
        cc_addr = []
    if bcc_addr is None:
        bcc_addr = []

    if type(to_addr) is str:
        to_addr = [to_addr]
    if type(cc_addr) is str:
        cc_addr = [cc_addr]
    if type(bcc_addr) is str:
        bcc_addr = [bcc_addr]

    if len(to_addr + cc_addr + bcc_addr) < 1:
        raise RecipientNotPresentException()

    return to_addr, cc_addr, bcc_addr


def _build_email(subject, text, to_emails, cc_emails, bcc_emails, attachment_path, html_text):
    global mail_cred
    if len(mail_cred.EMAIL_USER) > 0:
        sender = mail_cred.EMAIL_USER + ' <' + mail_cred.EMAIL_ID + '>'
    else:
        sender = mail_cred.EMAIL_ID
    message = MIMEMultipart()
    message['From'] = sender
    message['To'] = ",".join(to_emails)
    message['CC'] = ",".join(cc_emails)
    message['Subject'] = subject
    if text is not None:
        message.attach(MIMEText(text, 'plain'))
    if html_text is not None:
        message.attach(MIMEText(html_text, 'html'))

    if attachment_path is not None:
        attachment = open(attachment_path, 'rb')
        filename = os.path.basename(attachment_path)
        p = MIMEBase('application', 'octet-stream')
        p.set_payload((attachment).read())
        encoders.encode_base64(p)
        p.add_header('Content-Disposition', 'attachment; filename=%s' % filename)
        message.attach(p)

    toaddrs = to_emails + cc_emails + bcc_emails
    return toaddrs, message.as_string()



from notipyer import __url__ as url


class RecipientNotPresentException(Exception):
    def __init__(self):
        message = 'No recipients found.'
        super().__init__(message)


class GmailLoginException(Exception):
    def __init__(self):
        message = "The App password doesn't work." \
                 f" Please refer README at {url}"
        super().__init__(message)


class SlackApiException(Exception):
    def __init__(self, e):
        message = "The Slack trigger for the bot failed" \
                 f" with the error: {e.response}"
        super().__init__(message)



from .credential_handler import _set_slack_token_credentials
from .async_decorator import Async
from .errors import SlackApiException
from slack import WebClient
from slack.errors import SlackApiError
from notipyer import _get_creds

slack_creds = _get_creds()
slack_client = None


def set_slack_token_config(slack_token):
    global slack_creds
    _set_slack_token_credentials(slack_creds, slack_token)


def _get_slack_client():
    global slack_creds, slack_client
    if slack_client is None:
        slack_client = WebClient(token=slack_creds.SLACK_TOKEN)
    return slack_client


@Async
def send_message(channel, text):
    try:
        _get_slack_client().chat_postMessage(
            channel=channel,
            text=text
        )
    except SlackApiError as e:
        raise SlackApiException(e)



from .credential_handler import credentials

__version__ = '0.3.3'
__name__ = 'Notipyer'
__short_desc__ = 'Notification Triggers for Python'
__url__ = 'https://github.com/chirag-jn/notipyer'


creds = None


def _get_creds():
    global creds
    if creds is None:
        creds = credentials()
    return creds



from setuptools import setup, find_packages

from notipyer import __version__ as version
from notipyer import __url__ as url
from notipyer import __name__ as name
from notipyer import __short_desc__ as short_desc


with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name=name,
    version=version,
    description=short_desc,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Chirag Jain',
    url=url,
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6'
)



import threading


class TimeoutError(RuntimeError):
    pass


class AsyncCall(object):
    def __init__(self, fnc, callback=None):
        self.Callable = fnc
        self.Callback = callback

    def __call__(self, *args, **kwargs):
        self.Thread = threading.Thread(
            target=self.run,
            name=self.Callable.__name__,
            args=args, kwargs=kwargs)
        self.Thread.start()
        return self

    def wait(self, timeout=None):
        self.Thread.join(timeout)
        if self.Thread.isAlive():
            raise TimeoutError()
        else:
            return self.Result

    def run(self, *args, **kwargs):
        self.Result = self.Callable(*args, **kwargs)
        if self.Callback:
            self.Callback(self.Result)


class AsyncMethod(object):
    def __init__(self, fnc, callback=None):
        self.Callable = fnc
        self.Callback = callback

    def __call__(self, *args, **kwargs):
        return AsyncCall(self.Callable, self.Callback)(*args, **kwargs)


def Async(fnc=None, callback=None):
    if fnc is None:
        def AddAsyncCallback(fn):
            return AsyncMethod(fn, callback)

        return AddAsyncCallback
    else:
        return AsyncMethod(fnc, callback)
